﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using System.IO.IsolatedStorage;
using System.Reflection;
using System.Xml;
using Infragistics;
using Infragistics.Persistence;
using Infragistics.Windows.DataPresenter;


namespace StatePersistenceIdentifierSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            settings.Events.PersistenceSaved +=
                new System.EventHandler<PersistenceSavedEventArgs>(Events_PersistenceSaved);
            settings.Events.PersistenceLoaded += new EventHandler<PersistenceLoadedEventArgs>(_PersistenceLoaded);
           
        }

        private void _PersistenceLoaded(object sender, PersistenceLoadedEventArgs e)
        {
            listView.UpdateLayout();
        }

        private void Events_PersistenceSaved(object sender, PersistenceSavedEventArgs e)
        {
            IsSavedCompleted = true;
        }

        private PersistenceSettings settings = new PersistenceSettings();
        private MemoryStream memoryStream = new MemoryStream();
        public bool IsSavedCompleted;
        
        public MemoryStream Stream;
        private const string fileName = "testFile";
       
        private void BtnSave_Click(object sender, RoutedEventArgs e)
        {
            SaveIntoIsolatedStorage(listView, fileName);            
        }

        private void SaveIntoIsolatedStorage(DependencyObject obj, string fileName)
        {
            // Save DependencyObject properties into mMemoryStream
            MemoryStream memoryStream = PersistenceManager.Save(obj, settings);

            // Obtains user-scoped isolated storage and creates a file with persistence settings
            using (IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForAssembly())
            {
                if (iso.FileExists(fileName))
                    iso.DeleteFile(fileName);

                using (
                    IsolatedStorageFileStream stream = new IsolatedStorageFileStream(fileName, FileMode.OpenOrCreate,
                        iso))
                {
                    stream.Write(memoryStream.ToArray(), 0, (int) memoryStream.Length);
                    //string path = stream.GetType().GetField("m_FullPath", BindingFlags.Instance | BindingFlags.NonPublic).GetValue(stream).ToString();
                }

            }


        }

        private void BtnLoad_Click(object sender, RoutedEventArgs e)
        {
            LoadFromIsolatedStorage(listView, fileName);
            
        }

        private void LoadFromIsolatedStorage(DependencyObject obj, string fileName)
        {
            using (IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForAssembly())
            {
                if (iso.FileExists(fileName))
                {
                    using (
                        IsolatedStorageFileStream stream = new IsolatedStorageFileStream(fileName, FileMode.OpenOrCreate,
                            iso))
                    {
                        // loads persistence settings from IsolatedStorageFileStream into DependencyObject
                        PersistenceManager.Load(obj, stream, settings);
                    }
                }
            }
        }

        private void BtnRemove_OnClick(object sender, RoutedEventArgs e)
        {
            var categories = (DataContext as CategoryDataViewModel).Categories;
            categories.Clear();
            listView.UpdateLayout();
        }

        private void BtnTheme1_OnClick(object sender, RoutedEventArgs e)
        {
            Application.Current.Resources["WidgetLayoutRootBackground"] =
                (SolidColorBrush) (new BrushConverter().ConvertFrom("#FF5A5A5D"));
            Application.Current.Resources["WidgetTitleForeground"] =
                (SolidColorBrush) (new BrushConverter().ConvertFrom("#FFF5F5F5"));
            Application.Current.Resources["WidgetChartBackground"] =
                (SolidColorBrush) (new BrushConverter().ConvertFrom("#FF2D2D30"));
        }

        private void BtnTheme2_OnClick(object sender, RoutedEventArgs e)
        {

            Application.Current.Resources["WidgetLayoutRootBackground"] =
                (SolidColorBrush) (new BrushConverter().ConvertFrom("#FFBCD2EB"));
            Application.Current.Resources["WidgetTitleForeground"] =
                (SolidColorBrush) (new BrushConverter().ConvertFrom("#FF293955"));
            Application.Current.Resources["WidgetChartBackground"] =
                (SolidColorBrush) (new BrushConverter().ConvertFrom("#FFF2F2F2"));

        }
    }

    public static class Helper
        {
            public static T GetChildOfType<T>(this DependencyObject depObj) where T : DependencyObject
            {
                if (depObj == null) return null;

                for (int i = 0; i < VisualTreeHelper.GetChildrenCount(depObj); i++)
                {
                    var child = VisualTreeHelper.GetChild(depObj, i);

                    var result = (child as T) ?? GetChildOfType<T>(child);
                    if (result != null) return result;
                }
                return null;
            }
        }
    }
